/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.sensor.greenseeker;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.EnumMap;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import org.openide.util.NbBundle;

/**
 * FileProcessor for Normal type Greenseeker sensor files.
 * @author Arjan Loermans
 */
public class GreenseekerFileProcessorNormal extends GreenseekerFileProcessor {

    public GreenseekerFileProcessorNormal(GreenseekerDataObject toProcess) {
        super(toProcess);
    }
    // Constants    
    private static final SimpleDateFormat TIME_FORMAT = new SimpleDateFormat("HHmmss.SS");
    private static final String LOGKEY_TYPE_DISPLAYNAME = NbBundle.getMessage(GreenseekerFileProcessorNormal.class, "sensor.greenseeker.fileprocessor.normal.log.displayname");
    public static final int nrOfFields = 14;


    private static final AppLogger LOG = AppLogFactory.getLogger(GreenseekerFileProcessorNormal.class);

    /**
     * Parses one record of a normal Greenseeker sensorfile.
     *
     * @param payload The record to be processed.
     * @param dataFile The SensorDataFile belonging to this sensor file.
     * @return A record represented as a Greenseeker sensordata object
     * @throws ParseException Exception occurs when date/time cannot be parsed.
     */
    @Override
    protected GreenseekerSensorData parseLine(Object[] payload) throws GreenseekerParseException {

        // First create a entity representation of a file
        //converting the differrent data
        EnumMap map = new EnumMap<GreenseekerFileDescriptionNormal, Object>(GreenseekerFileDescriptionNormal.class);
        try {
            for (GreenseekerFileDescriptionNormal greenseekerNormalField : GreenseekerFileDescriptionNormal.values()) {
                if (greenseekerNormalField.isMandatory()) {
                    map.put(greenseekerNormalField, payload[greenseekerNormalField.getPosition()]);
                }
            }
        } catch (Exception e) {
            throw new GreenseekerParseException(e);
        }

        GreenseekerSensorData sensorData = new GreenseekerSensorData();

        sensorData.setObjectId((Double) map.get(GreenseekerFileDescriptionNormal.OBJECTID));
        sensorData.setLongitude((Double) map.get(GreenseekerFileDescriptionNormal.LONGITUDE));
        sensorData.setLatitude((Double) map.get(GreenseekerFileDescriptionNormal.LATITUDE));
        sensorData.setSpeed((Double) map.get(GreenseekerFileDescriptionNormal.SPEED));
        sensorData.setHeading((Double) map.get(GreenseekerFileDescriptionNormal.HEADING));
        sensorData.setElevation((Double) map.get(GreenseekerFileDescriptionNormal.ELEVATION));
        sensorData.setNdvi((Double) map.get(GreenseekerFileDescriptionNormal.NDVI));
        sensorData.setVi2nd((Double) map.get(GreenseekerFileDescriptionNormal.VI_2ND));

        sensorData.setSensor10((Double) map.get(GreenseekerFileDescriptionNormal.SENSOR_10));
        sensorData.setSensor19((Double) map.get(GreenseekerFileDescriptionNormal.SENSOR_19));
        sensorData.setSensor23((Double) map.get(GreenseekerFileDescriptionNormal.SENSOR_23));
        sensorData.setSensor24((Double) map.get(GreenseekerFileDescriptionNormal.SENSOR_24));
        sensorData.setSensor26((Double) map.get(GreenseekerFileDescriptionNormal.SENSOR_26));
        sensorData.setSensor73((Double) map.get(GreenseekerFileDescriptionNormal.SENSOR_73));

        return sensorData;
    }

    /**
     * Concrete subclasses must override this method to return the displayname
     * of that sensorprocessor to be displayed in the log. For example: "ASC-340
     * Cropcircle"
     *
     * Primary use is for logging, for example: [Info] Start processing of
     * ASC-340 Cropcircle sensor file 010710AD.CSV
     *
     * @return
     */
    @Override
    protected String getLogDisplayName() {
        return LOGKEY_TYPE_DISPLAYNAME;
    }
}
